-- name=mg_breakthrough
--*********************************************************************************************************************
function public.Init()

  --------------------------------------------------------------------------------------------
  private.room_objname = "mg_breakthrough";
  --------------------------------------------------------------------------------------------
  private.enemies = { 3, 4, 4, 5 };
  private.enemies_ways = { 3, 3, 4, 3 };
  private.hp_size = 248;
  private.damage_size = 0.125;
  private.jean_active = false;
  private.game_active = false;
  private.in_death = false;
  
  private.alien_type = { "purple", "green" };
  private.alien_act = { "waiting", "jump", "atack" };
  private.alien_type_attack = { "func_damage_from_purpur", "func_damage_from_green" };
  
  private.current_alien = { 0, 0, 0, 0 };
  
  private.enemy_pause = { 1.8, 1.5, 1.2, 0.8 };
  private.enemy_move = { {2, 1, 1}, {3, 2, 1}, {2, 1, 2, 3}, {1, 1, 1} };
  private.enemy_out = { {3, 2, 2}, {4, 3, 2}, {3, 2, 3, 4}, {0, 2, 2} };
  private.enemy_next = 0;
  
  private.holo_pos = { {436, 242}, {336, 135}, {345, 442} };
  
  private.map_hero_pos = { 175, 85, -5, -115 };
  
  private.enemy_event = { {0, 0, 0}, {"vent_2_1", "vent_2_2", 0}, {0, "vent_3_1", "vent_3_2", "vent_3_3"}, {0, 0, 0} };
  private.enemy_event_snd = { {0, 0, 0}, {"2nd_block_floor", "2nd_block_roof", 0}, {0, "3nd_block_3_hit", "3nd_block_1_hit", "3nd_block_1_hit"}, {0, 0, 0} };
  
  
  private.progress = "std";
  private.secret_al = { 0, 0, 0, 0 };
  --------------------------------------------------------------------------------------------
  -- function win_breakthrough () end;
  --------------------------------------------------------------------------------------------
  game.hint[ "win_breakthrough" ] = {
    type = "win",
    room = "rm_maindeck",
    zz = "mg_breakthrough",
    zz_gate = "gmg_maindeck_breakthrough"
  };
  --------------------------------------------------------------------------------------------
  local win_breakthrough = function ()

    if cmn.IsEventDone( "win_breakthrough" ) then
      ObjSet("obj_breakthrough_back",  { input = 0 });
    end;

  end;
  local win_breakthrough_loc = function ()

    if cmn.IsEventDone( "win_breakthrough" ) then
      ObjDelete("gmg_maindeck_breakthrough");
      ObjSet( "grm_maindeck_hangar", { input = 1, alp = 1 } );
      if port.isPortable() == 1 then
        port.GrmEventLightOn("grm_maindeck_hangar_focus");
      end;
    end;
    
  end;
  --------------------------------------------------------------------------------------------
  cmn.AddSubscriber( "win_breakthrough", win_breakthrough, "mg_breakthrough" );
  cmn.AddSubscriber( "win_breakthrough", win_breakthrough_loc, "rm_maindeck" );
  
end;
--*********************************************************************************************************************
function public.Load()

  if not ng_global.progress[ ng_global.currentprogress ][ "win_breakthrough" ].state or ng_global.currentprogress == "secret" then
    private.StartConditions();
  end;

  private.state = ng_global.progress[ ng_global.currentprogress ][ "win_breakthrough" ].state;
  private.events = ng_global.progress[ ng_global.currentprogress ][ "win_breakthrough" ].events;

end;
--*********************************************************************************************************************
function public.Open() 
  
  cmn.PlayAudio("mus", "assets/levels/level2/audio/mus/aud_mg_breakthrough_1", 1);
  
  if ng_global.currentprogress == "std" and interface.GetCurrentComplexInv() ~= "" then
    interface.ComplexInvHide();
  end;

  if ng_global.currentprogress == "std" and not(cmn.IsEventDone( "win_breakthrough" )) then
    cmn.MiniGameShow();
    private.CheckWin();
    cmn.StartTimePassing("mg_breakthrough");
  elseif ng_global.currentprogress == "secret" then
    rm_extra.MiniGameShow();
  end;

end;
--*********************************************************************************************************************
function public.PreOpen()
  
  if private.state[1] == 0 then
    cmn.StopAllEnv("assets/levels/level2/audio/env/aud_rm_maindeck_env");
    cmn.PlayAudio("env", "assets/levels/level2/audio/env/aud_rm_maindeck_env");
  else 
    cmn.StopAllEnv();
  end;

  if ng_global.currentprogress == "std" then
    private.progress = "std";
    common.SetPreviousRoom( "rm_maindeck" );
  else
    
    SetCursor(CURSOR_NULL);
    
    private.progress = "secret";
    private.StartConditions();
    
    private.jean_active = false;
    private.game_active = false;
    private.in_death = false;
    
    private.state = ng_global.progress[ ng_global.currentprogress ][ "win_breakthrough" ].state;
    private.events = ng_global.progress[ ng_global.currentprogress ][ "win_breakthrough" ].events;
    
    private.state[3] = ng_global.player_gun;
    
    ObjMultiSet( { {"obj_breakthrough_holo", { input = 0 }},
                   {"anm_breakthrough_holo", { alp = 0 }},
                   {"anm_breakthrough_jean_"..private.state[3], { animfunc = "func_start_mg_"..private.state[3], playing = 0, frame = 0 }},
                   {"anm_breakthrough_hatch", { alp = 1, frame = 0 }},
                   {"spr_breakthrough_hatch_lock", { alp = 1 }},
                   {"anm_breakthrough_door_open", { frame = 0 }},
                   {"obj_breakthrough_level_4", { alp = 0, input = 0, pos_y = 384, scale_x = 1, scale_y = 1 }},
                   {"obj_breakthrough_back", { input = 1 }} } );
    
    ObjSet("grm_breakthrough_maindeck",  { event_mdown = "", event_mup = rm_extra.GoToSecretroom, input = 1,
                                         event_menter = function () rm_extra.GateMouseEnter( "breakthrough_maindeck", CURSOR_DOWN, "extra" ); end, 
                                         event_mleave = function () common_impl.GateMouseLeave( "breakthrough_maindeck" ); end });
                                     
  end;
    
  private.DrawScene();
  
end;
--*********************************************************************************************************************
function public.Close()
  
  SetCursor(CURSOR_DEFAULT);
  
  if private.progress == "secret" then
    
    local stage = ng_global.progress[ "secret" ][ "win_breakthrough" ].state[1];
    local count = ng_global.progress[ "secret" ][ "win_breakthrough" ].state[2];
    
    local al = private.secret_al[1];
    local way = private.secret_al[3];
    local con = private.secret_al[4];
    
    --private.StopAlien();
    private.enemy_next = 0;
    
    if al > 0 then
      private.ResetAlien( stage, way, con );
    end;
    
    --ObjSet("tmr_breakthrough_timer_alien",  { playing = 0 });
    
  end;
  
end;
--*********************************************************************************************************************
function public.PreClose()
  
  SndStop("assets/levels/level2/audio/sfx/aud_mmg_monsters_appears", 0.5);
  
  cmn.StopAllMus();
  
  if ng_global.currentprogress == "std" then
    common_impl.PlayBackgroundMusic();
  end;

  if ng_global.currentprogress == "std" and not(cmn.IsEventDone( "win_breakthrough" )) then
    cmn.MiniGameHide();
  elseif ng_global.currentprogress == "secret" and not(cmn.IsEventDone( "win_breakthrough" )) then
    rm_extra.MiniGameHide();
  end;
  
  if private.progress == "secret" then
    private.secret_al = { 0, 0, 0, 0 };
    private.secret_al[1] = private.current_alien[1];
    private.secret_al[3] = private.current_alien[3];
    private.secret_al[4]= private.current_alien[4];
    private.StopAlien();
    ObjSet("tmr_breakthrough_timer_alien",  { playing = 0 });
  end;
  
end;
--*********************************************************************************************************************
function public.Reset()
  
  if private.state[1] == 0 then
    
    if private.state[2] > 0 then
      
      cmn.Lock( 1 );
    
      private.state[2] = 0;
      ObjAnimate("anm_breakthrough_holo", "alp", 0, 0, "", { 0, 0, 1,   0.5, 0, 0 });
      
      ObjMultiSet( { {"anm_breakthrough_jean_"..private.state[3], { animfunc = "func_jim_stop_hologram_"..private.state[3], alp = 1, frame = 0, playing = 1, endtrig = private.SetJean }},
                     {"obj_breakthrough_holo", { input = 0 }},
                     {"txt_breakthrough_frags_count", { text = "0/3" }} } );
                     
    end;
    
  else 
                   
    cmn.Lock( 1 );
    SndStop("assets/levels/level2/audio/sfx/aud_mmg_monsters_appears", 0.5);
                   
    local stage = private.state[1];
    local count = private.state[2];
    
    local al = private.current_alien[1];
    local way = private.current_alien[3];
    local con = private.current_alien[4];
    
    --DbgTrace(al)
    
    private.StopAlien();
    private.enemy_next = 0;
    
    if al > 0 then
      local reset_alien = function ()
        private.ResetAlien( stage, way, con );
      end;
      ObjAnimate("anm_breakthrough_"..stage.."_alien_"..way.."_"..con.."_state", "alp", 0, 0, reset_alien, { 0, 0, 1,   0.4, 0, 0 });
    end;
    
    if stage > 1 then
      
      --cmn.Lock( 1 );
      private.NewLoc();
      
    elseif private.state[2] > 0 or private.state[4] < 8 then
      
      --cmn.Lock( 1 );
      
      if private.state[2] > 0 then
      
        private.state[2] = 0;
        ObjSet("txt_breakthrough_frags_count", { text = "0/"..private.enemies[1] });
        
      end;
      
      if private.state[4] < 8 then
        
        local hp = private.state[4];
        private.state[4] = 8;
        
        local old = -math.floor(private.hp_size*(hp+1)*private.damage_size);
        local new = -math.floor(private.hp_size*private.state[4]*private.damage_size);
        ObjAnimate("spr_breakthrough_health", "croprect_y", 0, 0, "", { 0, 0, old,   0.5, 0, new });
        
      end;
      
      ObjSet("tmr_breakthrough_timer_alien",  { time = 0.5, playing = 1, endtrig = function() private.SetAlien(); cmn.Lock( 0 ); end });
      
    else 
      ObjSet("tmr_breakthrough_timer_alien",  { time = 0.5, playing = 1, endtrig = function() private.SetAlien(); cmn.Lock( 0 ); end });
    end;
    
  end;

end;
--*********************************************************************************************************************
function public.Skip()
  
  cmn.Lock( 1 );
  SndStop("assets/levels/level2/audio/sfx/aud_mmg_monsters_appears", 0.5);
  
  ng_global.progress[ ng_global.currentprogress ][ "win_breakthrough" ].skip = 1;
  ng_global.progress[ ng_global.currentprogress ][ "win_breakthrough" ].state = {4, 5, 1, 8};
  
  local gun = private.state[3];
  local hp = private.state[4];
  
  ng_global.progress[ ng_global.currentprogress ][ "win_breakthrough" ].state[3] = gun;
  ng_global.progress[ ng_global.currentprogress ][ "win_breakthrough" ].state[4] = hp;
  
  local stage = private.state[1];
  local count = private.state[2];
  
  if private.current_alien[1] > 0 then
    ObjAnimate("anm_breakthrough_"..stage.."_alien_"..private.current_alien[3].."_"..private.current_alien[4].."_state", "alp", 0, 0, "", { 0, 0, 1,   0.5, 0, 0 });
  end;
  
  private.StopAlien();
  
  private.state[1] = 4;
  private.state[2] = 5;
  
  if stage < 4 then
    
    local anm_next = function ()
      
      ObjAnimate("spr_breakthrough_dark", "alp", 0, 0, private.CheckWin, { 0, 0, 1,   0.5, 0, 0 });
      
      for s = stage+1, 4, 1 do
        ObjSet("spr_breakthrough_map_level_"..s,  { alp = 1 });
      end;
      
      ObjMultiSet( { {"obj_breakthrough_level_4", { alp = 1, input = 0 }},
                     {"obj_breakthrough_level_"..stage, { alp = 0, input = 0 }},
                     {"spr_breakthrough_map_hero",  { pos_y = private.map_hero_pos[ 4 ] }} } );
      
    end;
    
    ObjAnimate("spr_breakthrough_dark", "alp", 0, 0, anm_next, { 0, 0, 0,   0.5, 0, 1 });
    
  else 
    
    ObjSet("tmr_breakthrough_timer_alien",  { time = 0.5, playing = 1, endtrig = private.CheckWin });
    
  end;

end;
--*********************************************************************************************************************
-- function *** MG *** () end;
--*********************************************************************************************************************
function private.StartConditions()

  ng_global.progress[ ng_global.currentprogress ][ "win_breakthrough" ].skip = 0;
  ng_global.progress[ ng_global.currentprogress ][ "win_breakthrough" ].state = { 0, 0, 1, 8 }; 
  --[1] - level, [2] - killed enemies, [3] - type of weapon, [4] - health
  
  ng_global.progress[ ng_global.currentprogress ][ "win_breakthrough" ].events = {
    ["vent_2_1"] = 0,
    ["vent_2_2"] = 0,
    ["vent_3_1"] = 0,
    ["vent_3_2"] = 0,
    ["vent_3_3"] = 0
  };

end;
-----------------------------------------------------------------------------------
function private.DrawScene()
  
  if private.state[1] == 0 and not private.jean_active and private.state[2] < 3 then
    private.SetJean();
  elseif private.state[1] == 0 and not private.jean_active and private.state[2] == 3 then
    
    if private.state[3] == 1 then
      ObjMultiSet( { {"anm_breakthrough_jean_1", { alp = 1 }},
                     {"anm_breakthrough_jean_2", { alp = 0 }} } );
    else 
      ObjMultiSet( { {"anm_breakthrough_jean_2", { alp = 1 }},
                     {"anm_breakthrough_jean_1", { alp = 0 }} } );
    end;            
    
  end;
                 
  for i = 2, 3, 1 do
    for j = 1, 3, 1 do
      
      if i == 3 and private.events[ "vent_"..i.."_"..j ] == 1 then
        ObjSet("anm_breakthrough_vent_"..i.."_"..j,  { alp = 0, frame = 0 });
      elseif i == 3 then
        ObjSet("anm_breakthrough_vent_"..i.."_"..j,  { alp = 1, frame = 0 });
      end;
      
      if i == 2 and j == 1 then
        if private.events[ "vent_"..i.."_"..j ] == 1 then
          ObjSet("anm_breakthrough_vent_"..i.."_"..j,  { alp = 0, animfunc = "func_lvl_2_floor_cover_1", frame = 0 });
        else 
          ObjSet("anm_breakthrough_vent_"..i.."_"..j,  { alp = 1, animfunc = "func_lvl_2_floor_cover_1", frame = 0 });
        end;
      end;
      
      if i == 2 and j == 2 then
        --DbgTrace(private.events[ "vent_"..i.."_"..j ])
        if private.events[ "vent_"..i.."_"..j ] == 1 then
          ObjMultiSet( { {"anm_breakthrough_vent_"..i.."_"..j, { alp = 0, frame = 0 }},
                         {"anm_breakthrough_vent_"..i.."_"..j.."_end", { alp = 1, playing = 1 }} } );
        else 
          ObjMultiSet( { {"anm_breakthrough_vent_"..i.."_"..j, { alp = 1, frame = 0 }},
                         {"anm_breakthrough_vent_"..i.."_"..j.."_end", { alp = 0, playing = 0 }} } );
        end;
      end;
      
    end;
  end;
  
  for i = 0, 4, 1 do
    
    if i == private.state[1] then
      ObjSet("obj_breakthrough_level_"..i,  { alp = 1, input = 1 });
    else 
      ObjSet("obj_breakthrough_level_"..i,  { alp = 0, input = 0 });
    end;
    
  end;
  
  for i = 1, private.state[1], 1 do
    ObjSet("spr_breakthrough_map_level_"..i,  { alp = 1 });
  end;
  
  for i = private.state[1]+1, 4, 1 do
    ObjSet("spr_breakthrough_map_level_"..i,  { alp = 0 });
  end;
  
  if private.state[1] > 0 then
    ObjMultiSet( { {"spr_breakthrough_map_hero",  { pos_y = private.map_hero_pos[ private.state[1] ] }},
                   {"obj_breakthrough_map", { alp = 1 }} } );
  else 
    ObjMultiSet( { {"spr_breakthrough_map_hero",  { pos_y = private.map_hero_pos[ 1 ] }},
                   {"obj_breakthrough_map", { alp = 0 }} } );
  end;
  
  for i = 1, 2, 1 do
    
    if i == private.state[3] then
      ObjSet("anm_breakthrough_gunfire_"..i,  { alp = 1 });
    else 
      ObjSet("anm_breakthrough_gunfire_"..i,  { alp = 0 });
    end;
    
  end;
  
  if private.state[4] == 0 then
    private.in_death = true;
  end;
  
  if private.state[1] == 0 then
    ObjSet("txt_breakthrough_frags_count",  { text = private.state[2].."/3" });
  else 
    
    if not private.game_active and not private.in_death then
      private.game_active = true;
      private.SetAlien();
    else 
      PlaySfx("assets/levels/level2/audio/sfx/aud_mmg_monsters_appears", 1, "", 1);
    end;
    
    ObjSet("txt_breakthrough_frags_count",  { text = private.state[2].."/"..private.enemies[ private.state[1] ] });
  end;
  
  ObjSet("spr_breakthrough_health",  { croprect_y = -math.floor(private.hp_size*private.state[4]*private.damage_size) });
  
  private.AimTarget();

end;
-----------------------------------------------------------------------------------
function private.SetJean()
  
  private.jean_active = true;
  local set_num = private.state[3];

  if set_num == 2 then
    ObjSet("anm_breakthrough_jean_1",  { alp = 0 });
  else 
    ObjSet("anm_breakthrough_jean_2",  { alp = 0 });
  end;
  
  local anm_start_3 = function ()
    
    cmn.Lock( 0 );

    ObjMultiSet( { {"anm_breakthrough_holo", { animfunc = "func_hologram_line", frame = 0, playing = 1 }},
                   {"obj_breakthrough_holo", { input = 1 }} } );
    
  end;
  
  local anm_start_2 = function ()
    
    ObjMultiSet( { {"anm_breakthrough_holo", { animfunc = "func_hologram_start", alp = 1, frame = 0, playing = 1, endtrig = anm_start_3 }},
                   {"anm_breakthrough_jean_"..set_num, { animfunc = "func_jim_waiting_with_hologram_"..set_num, alp = 1, frame = 0, playing = 1 }},
                   {"obj_breakthrough_holo", { pos_x = private.holo_pos[ private.state[2]+1 ][1], pos_y = private.holo_pos[ private.state[2]+1 ][2] }} } );
    
  end;
  
  local anm_start_1 = function ()
    
    ObjSet("anm_breakthrough_jean_"..set_num,  { animfunc = "func_jim_start_hologram_"..set_num, frame = 0, playing = 1, endtrig = anm_start_2 });
    
  end;
  
  cmn.Lock( 1 );
  cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_mmg_monsters_create_monster");
  
  ObjSet("anm_breakthrough_jean_"..set_num,  { animfunc = "func_start_mg_"..set_num, alp = 1, frame = 0, playing = 1, endtrig = anm_start_1 });
  
end;
-----------------------------------------------------------------------------------
function private.AimTarget()
  
  local curs = GetZoomCursorPos();
  curs[1] = math.floor(curs[1]);
  curs[2] = math.floor(curs[2]);
  
  local weap_pos = {0, 0};
  weap_pos[1] = curs[1];
  weap_pos[2] = curs[2]+220;
  
  if weap_pos[1] < 0 then
    weap_pos[1] = 0;
  elseif weap_pos[1] > 900 then
    weap_pos[1] = 900;
  end;
  
  if weap_pos[2] < 608 then
    weap_pos[2] = 608;
  elseif weap_pos[2] > 800 then
    weap_pos[2] = 800;
  end;
  
  ObjMultiSet( { {"obj_breakthrough_aim", { pos_x = curs[1], pos_y = curs[2] }},
                 {"obj_breakthrough_weapon", { pos_x = weap_pos[1], pos_y = weap_pos[2] }},
                 {"tmr_breakthrough_timer_aim", { time = 0.01, playing = 1, endtrig = private.AimTarget }} } );
  
end;
-----------------------------------------------------------------------------------
function private.SetAlien()
  
  --DbgTrace("SetAlien")
  
  local anm_con = function ()
    
    local ways = private.enemy_move[ private.state[1] ][ private.current_alien[3] ];
    local new_way = 0;
    
    if private.current_alien[1] > 0 then
      
      if ways ~= 0 then
        new_way = 1 + math.random(1, ways);
      end;
      
      --DbgTrace(new_way.." "..private.current_alien[3])
    
      if ways == 0 or private.enemy_out[ private.state[1] ][ private.current_alien[3] ] == new_way then
        
        --DbgTrace("pause "..private.current_alien[3])
        
        private.enemy_next = new_way;
        --DbgTrace(private.enemy_next)
        
        --if private.state[1] == 1 then
        --  private.enemy_next = private.enemy_out[ private.state[1] ][ private.current_alien[3] ];
        --end;
      
        if private.current_alien[2] > 1 then
          private.current_alien[2] = 1;
          ObjSet("anm_breakthrough_"..private.state[1].."_alien_"..private.current_alien[3].."_1_state",  { frame = 0, animfunc = "func_alien_"..private.alien_type[ private.current_alien[1] ].."_"..private.alien_act[ 1 ], playing = 1 });
        end;
        
        ObjMultiSet( { {"obj_breakthrough_"..private.state[1].."_alien_"..private.current_alien[3].."_1",  { input = 1 }},
                       {"tmr_breakthrough_timer_alien", { time = private.enemy_pause[ private.state[1] ], playing = 1, endtrig = function() private.AlienAttack( 1 ); end }} } );
        
      else 
        
        private.current_alien[4] = new_way;
        private.current_alien[2] = 1;
        private.ResetAlien( private.state[1], private.current_alien[3], 1 );
        private.enemy_next = 0;
        
        ObjMultiSet( { {"anm_breakthrough_"..private.state[1].."_alien_"..private.current_alien[3].."_"..private.current_alien[4], { alp = 1, frame = 0, playing = 1, endtrig = function() private.AlienAttack( private.current_alien[4] ); end }},
                       {"obj_breakthrough_"..private.state[1].."_alien_"..private.current_alien[3].."_"..private.current_alien[4],  { input = 1 }},
                       {"anm_breakthrough_"..private.state[1].."_alien_"..private.current_alien[3].."_"..private.current_alien[4].."_state", { alp = 1, frame = 0, animfunc = "func_alien_"..private.alien_type[ private.current_alien[1] ].."_"..private.alien_act[ private.current_alien[2] ], playing = 1 }} } );
        
      end;
                     
    end;
    
  end;
  
  if private.current_alien[1] == 0 then
  
    private.current_alien[1] = math.random(1, 2);
    private.current_alien[2] = math.random(1, 2);
    private.current_alien[3] = math.random(1, private.enemies_ways[ private.state[1] ]);
    private.current_alien[4] = 1;
    
  end;
  
  local start_alien = function ()
    
    --DbgTrace(private.state[1].." "..private.current_alien[3])
    
    cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_scare_string_monster_"..math.random(1,5));
    PlaySfx("assets/levels/level2/audio/sfx/aud_mmg_monsters_appears", 1, "", 1);
    
    ObjMultiSet( { {"anm_breakthrough_"..private.state[1].."_alien_"..private.current_alien[3].."_1", { alp = 1, frame = 0, playing = 1, endtrig = anm_con }},
                   {"anm_breakthrough_"..private.state[1].."_alien_"..private.current_alien[3].."_1_state", { alp = 1, frame = 0, animfunc = "func_alien_"..private.alien_type[ private.current_alien[1] ].."_"..private.alien_act[ private.current_alien[2] ], playing = 1 }},
                   {"obj_breakthrough_"..private.state[1].."_alien_"..private.current_alien[3].."_1",  { input = 1 }} } );
    
  end;
                 
  local event = private.enemy_event[ private.state[1] ][ private.current_alien[3] ];
  
  if event == 0 or private.events[ event ] == 1 then
    start_alien();
  else
    
    local event_after = function ()
      
      if private.state[1] == 3 then
        start_alien();
      else 
        
        if event == "vent_2_1" then
          ObjSet("anm_breakthrough_"..event,  { frame = 0, animfunc = "func_lvl_2_floor_cover_2", playing = 1, endtrig = start_alien });
        else 
          start_alien();
          ObjMultiSet( { {"anm_breakthrough_"..event, { alp = 0, frame = 0 }},
                         {"anm_breakthrough_"..event.."_end", { alp = 1, playing = 1 }} } );
        end;
        
      end;
      
    end;
    
    private.events[ event ] = 1;
    
    cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_mmg_monsters_"..private.enemy_event_snd[ private.state[1] ][ private.current_alien[3] ]);
    
    ObjSet("anm_breakthrough_"..event,  { playing = 1, endtrig = event_after });
    
  end;
  
end;
-----------------------------------------------------------------------------------
function private.AlienAttack( num )
  
  cmn.Lock( 1 );
  SndStop("assets/levels/level2/audio/sfx/aud_mmg_monsters_appears", 0.5);
  
  local stage = private.state[1];
  local way = private.current_alien[3];
  local damage = 1;
  local num2 = 0;
  
  if private.current_alien[1] == 2 then
    damage = 2;
  end;
  
  private.current_alien[2] = 3;
  private.state[4] = private.state[4]-damage;
  
  if private.state[4] < 0 then
    private.state[4] = 0;
  end;
  
  local alien = private.current_alien[1];
  
  local func_after = function ()
    
    ObjSet("anm_breakthrough_damage_"..alien,  { alp = 0, frame = 0 });
    
    private.ResetAlien( stage, way, num );
    
    if num2 > 0 then
      private.ResetAlien( stage, way, num2 );
    end;
    
    if private.state[4] == 0 then
      private.HeroDeath();
    else 
      cmn.Lock( 0 );
      --ObjAnimate("anm_breakthrough_damage", "alp", 0, 0, function() private.CheckWin(true); end, { 0, 0, 1,   0.3, 0, 0 });
      private.CheckWin(true);
    end;
    
  end;
  
  local anm_damage = function ()
    
    private.StopAlien();
    
    local old = -math.floor(private.hp_size*(private.state[4]+1)*private.damage_size);
    local new = -math.floor(private.hp_size*private.state[4]*private.damage_size);
    ObjAnimate("spr_breakthrough_health", "croprect_y", 0, 0, "", { 0, 0, old,   0.5, 0, new });
    
    --DbgTrace(private.enemy_out[ private.state[1] ][ way ])
    --DbgTrace(private.enemy_next)
    
    if private.enemy_out[ private.state[1] ][ way ] == private.enemy_next and private.enemy_out[ private.state[1] ][ way ] > 0 then
      
      --num2 = private.enemy_next;
      num2 = private.enemy_out[ private.state[1] ][ way ];
      private.enemy_next = 0;
      
      ObjMultiSet( { {"anm_breakthrough_"..stage.."_alien_"..way.."_"..num2.."_state",  { frame = 0, animfunc = "func_alien_"..private.alien_type[ alien ].."_"..private.alien_act[ 1 ], playing = 1 }},
                     {"anm_breakthrough_"..stage.."_alien_"..way.."_"..num2, { alp = 1, frame = 0, playing = 1, endtrig = func_after }},
                     {"anm_breakthrough_"..stage.."_alien_"..way.."_"..num, { alp = 0 }} } );
                     
    else
      private.enemy_next = 0;
      ObjAnimate("anm_breakthrough_"..stage.."_alien_"..way.."_"..num.."_state", "alp", 0, 0, func_after, { 0, 0, 1,   0.5, 0, 0 });
    end;
    
    --ObjSet("anm_breakthrough_damage_"..alien,  { alp = 1, playing = 1, endtrig = func_after });
    --ObjAnimate("anm_breakthrough_damage", "alp", 0, 0, func_after, { 0, 1, 0,   0.2, 1, 1,  0.4, 1, 0,   0.6, 1, 1,  0.8, 1, 0,    1, 1, 1,   1.2, 1, 0,   1.5, 1, 1 });
    
  end;
  
  if alien == 1 then
    cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_mmg_monsters_monster_hits");
  else 
    cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_mmg_monsters_spill_acid");
  end;
  
  ObjMultiSet( { {"obj_breakthrough_"..stage.."_alien_"..way.."_"..num,  { input = 0 }},
                 {"anm_breakthrough_"..stage.."_alien_"..way.."_"..num.."_state",  { frame = 0, animfunc = "func_alien_"..private.alien_type[ private.current_alien[1] ].."_"..private.alien_act[ private.current_alien[2] ].."_1", playing = 1, endtrig = anm_damage }},
                 {"anm_breakthrough_damage_"..alien,  { alp = 1, playing = 1 }} } );
  
end;
-----------------------------------------------------------------------------------
function private.ResetAlien( stage, way, con )
  
  ObjMultiSet( { {"anm_breakthrough_"..stage.."_alien_"..way.."_"..con, { alp = 0, frame = 0, playing = 0, endtrig = "" }},
                 {"anm_breakthrough_"..stage.."_alien_"..way.."_"..con.."_state", { alp = 1, frame = 0, playing = 0, endtrig = "" }},
                 {"obj_breakthrough_"..stage.."_alien_"..way.."_"..con, { input = 0 }} } );
  
end;
-----------------------------------------------------------------------------------
function private.StopAlien()
  
  local stage = private.state[1];
  local way = private.current_alien[3];
  local con = private.current_alien[4];
  
  private.current_alien = { 0, 0, 0, 0 };
  
  ObjMultiSet( { {"anm_breakthrough_"..stage.."_alien_"..way.."_"..con, { playing = 0, endtrig = "" }},
                 {"anm_breakthrough_"..stage.."_alien_"..way.."_"..con.."_state", { playing = 0, endtrig = "" }},
                 {"tmr_breakthrough_timer_alien", { playing = 0 }} } );
  
end;
-----------------------------------------------------------------------------------
function private.WeaponBlock( param, nolock )
  
  nolock = nolock or false;
  
  --if not nolock then
    local check = 1;
    if (param == 1) then
      check = 0;
    end;
    ObjSet("mg_breakthrough",{ input = check });
    --cmn.Lock( param );
  --end;
  
  ObjSet("spr_breakthrough_aim_2",  { alp = param });
  
end;
-----------------------------------------------------------------------------------
function public.ShootHolo()
  
  cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_mmg_monsters_catch");
  
  private.state[2] = private.state[2]+1;
  ObjSet("txt_breakthrough_frags_count",  { text = private.state[2].."/3" });
  
  private.WeaponBlock( 1 );
  
  ObjMultiSet( { {"anm_breakthrough_gunfire_"..private.state[3],  { playing = 1, endtrig = function() private.WeaponBlock(0, true); end }},
                 {"anm_breakthrough_holo", { animfunc = "func_hologram_die", alp = 1, frame = 0, playing = 1, endtrig = function() private.CheckWin( true ); end }},
                 {"obj_breakthrough_holo", { input = 0 }} } );
  
end;
-----------------------------------------------------------------------------------
function public.MissHit()
  
  --DbgTrace("MissHit");
  cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_mmg_monsters_shot");
  
  private.WeaponBlock( 1 );
  
  ObjSet("anm_breakthrough_gunfire_"..private.state[3],  { playing = 1, endtrig = function() private.WeaponBlock(0); end });
  
end;
-----------------------------------------------------------------------------------
function public.HitTarget( num )
  
  cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_mmg_monsters_catch");
  SndStop("assets/levels/level2/audio/sfx/aud_mmg_monsters_appears", 0.5);
  
  private.state[2] = private.state[2]+1;
  
  local stage = private.state[1];
  local way = private.current_alien[3];
  local block = false;
  
  if private.state[2] == private.enemies[ private.state[1] ] then
    block = true;
  end;
  
  ObjSet("txt_breakthrough_frags_count",  { text = private.state[2].."/"..private.enemies[ private.state[1] ] });
  
  --DbgTrace("hit target "..num);
  private.StopAlien();
  
  local func_after = function ()
    ObjDetach("spr_breakthrough_alien_dead");
    private.ResetAlien( stage, way, num );
    private.CheckWin(true);
  end;
  
  private.WeaponBlock( 1 );
  
  ObjAttach("spr_breakthrough_alien_dead", "obj_breakthrough_"..stage.."_alien_"..way.."_"..num);
  
  ObjAnimate("anm_breakthrough_"..stage.."_alien_"..way.."_"..num.."_state", "alp", 0, 0, "", { 0, 0, 1,   0.5, 0, 0 });
  ObjAnimate("spr_breakthrough_alien_dead", "alp", 0, 0, func_after, { 0, 0, 0,   0.5, 0, 1,   1, 0, 0 });
  
  ObjSet("anm_breakthrough_gunfire_"..private.state[3],  { playing = 1, endtrig = function() private.WeaponBlock(0, block); end });
  
end;
-----------------------------------------------------------------------------------
function private.NewLoc()
  
  local func_after = function ()
    
    private.state[1] = 1;
    private.state[2] = 0;
    private.state[4] = 8;
    
    private.game_active = true;
    private.DrawScene();
    
    local end_func = function ()
      cmn.Lock(0);
      private.SetAlien();
      ObjSet("anm_breakthrough_gates_open",  { alp = 0, frame = 0 });
    end;
    
    local anm_door = function ()
      cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_mmg_monsters_door_opens");
      ObjSet("anm_breakthrough_gates_open", { playing = 1, endtrig = end_func });
    end;
    
    ObjSet("anm_breakthrough_gates_open", { alp = 1 });
    ObjAnimate("spr_breakthrough_dark", "alp", 0, 0, anm_door, { 0, 0, 1,   0.5, 0, 0 });
    
  end;
  
  ObjAnimate("spr_breakthrough_dark", "alp", 0, 0, func_after, { 0, 0, 0,   0.5, 0, 1 });
  
end;
-----------------------------------------------------------------------------------
function private.HeroDeath()
  
  private.in_death = true;
  
  local func_after = function ()
    
    private.state[1] = 1;
    private.state[2] = 0;
    private.state[4] = 8;
    
    private.game_active = true;
    private.in_death = false;
    private.DrawScene();

    ObjMultiSet( { {"anm_breakthrough_damage", { alp = 0 }},
                   {"anm_breakthrough_die", { alp = 0, frame = 0 }},
                   {"anm_breakthrough_restart", { alp = 1 }} } );
    
    local end_func = function ()
      cmn.Lock(0);
      private.SetAlien();
      ObjSet("anm_breakthrough_gates_open",  { alp = 0, frame = 0 });
    end;
    
    local anm_door = function ()
      cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_mmg_monsters_door_opens");
      ObjAnimate("obj_breakthrough_interface", "alp", 0, 0, "", { 0, 0, 0,   0.3, 0, 1 });
      ObjAnimate("obj_breakthrough_interface_2", "alp", 0, 0, "", { 0, 0, 0,   0.3, 0, 1 });
      ObjMultiSet( { {"anm_breakthrough_restart",  { alp = 0, frame = 0 }},
                     {"anm_breakthrough_gates_open", { alp = 1, playing = 1, endtrig = end_func }} } );
    end;
    
    local anm_restart = function ()
      ObjSet("anm_breakthrough_restart",  { playing = 1, endtrig = anm_door });
    end;
    
    anm_restart();
    --ObjAnimate("spr_breakthrough_dark", "alp", 0, 0, anm_restart, { 0, 0, 1,   0.3, 0, 0 });
    
  end;
  
  local complete_dark = function ()
    --ObjAnimate("spr_breakthrough_dark", "alp", 0, 0, func_after, { 0, 0, 0,   0.3, 0, 1 });
    ObjAnimate("obj_breakthrough_interface", "alp", 0, 0, func_after, { 0, 0, 1,   0.3, 0, 0 });
    ObjAnimate("obj_breakthrough_interface_2", "alp", 0, 0, "", { 0, 0, 1,   0.3, 0, 0 });
  end;
  
  cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_anom_eye_close");
  
  ObjSet("anm_breakthrough_die",  { alp = 1, frame = 0, playing = 1, endtrig = complete_dark });
  
end;
-----------------------------------------------------------------------------------
function private.CheckWin( ingame )
  
  ingame = ingame or false;
  
  if private.state[1] > 0 and private.state[2] == private.enemies[ private.state[1] ] then
    
    cmn.Lock( 1 );
    
    if private.state[1] == 4 then
      private.Win();
    else 
      
      local old = private.state[1];
      
      local set_new_room = function ()
        
        --ObjSet("txt_breakthrough_frags_count",  { text = private.state[2].."/"..private.enemies[ private.state[1] ] });
        ObjMultiSet( { {"txt_breakthrough_frags_count",  { text = private.state[2].."/"..private.enemies[ private.state[1] ] }},
                       {"spr_breakthrough_map_level_"..private.state[1], { alp = 1 }},
                       {"spr_breakthrough_map_hero",  { pos_y = private.map_hero_pos[ private.state[1] ] }} } );
        
        local anm_end = function ()
          cmn.Lock( 0 );
          private.SetAlien();
          ObjSet("anm_breakthrough_gates_open",  { frame = 0, alp = 0 });
        end;
    
        local anm_doors = function ()
          cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_mmg_monsters_door_opens");
          ObjSet("anm_breakthrough_gates_open",  { playing = 1, endtrig = anm_end });
        end;
        
        ObjAnimate("spr_breakthrough_dark", "alp", 0, 0, anm_doors, { 0, 0, 1,   0.5, 0, 0 });
          
        ObjMultiSet( { {"obj_breakthrough_level_"..old, { alp = 0, input = 0, pos_y = 384, scale_x = 1, scale_y = 1 }},
                       {"obj_breakthrough_level_"..private.state[1], { alp = 1, input = 1 }},
                       {"obj_breakthrough_interface", { pos_y = 384, scale_x = 1, scale_y = 1 }},
                       {"anm_breakthrough_gates_open", { alp = 1 }} } );

      end;
      
      private.state[1] = private.state[1] + 1;
      private.state[2] = 0;
      
      cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_mmg_monsters_steps");
      
      ObjAnimate( "obj_breakthrough_level_"..old, "scale_xy", 0, 0, "",
      {
        0.0, 3, 1, 1,
        1.6, 3, 1.3, 1.3
      } );
      
      ObjAnimate( "obj_breakthrough_level_"..old, "pos_y", 0, 0, "",
      {
        0.0, 3, 384,
        0.4, 3, 374,
        0.8, 3, 394,
        1.2, 3, 374,
        1.6, 3, 394
      } );
      
      ObjAnimate( "obj_breakthrough_interface", "pos_y", 0, 0, "",
      {
        0.0, 3, 384,
        0.4, 3, 374,
        0.8, 3, 394,
        1.2, 3, 374,
        1.6, 3, 394
      } );
      
      ObjAnimate("spr_breakthrough_dark", "alp", 0, 0, set_new_room, { 0, 0, 0,   1.4, 0, 0,   1.8, 0, 1 });
      
    end;
    
  elseif private.state[1] == 0 and private.state[2] == 3 then
    
    cmn.Lock( 1 );
    cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_mmg_monsters_girl_shoot_down");
    
    --private.state[1] = private.state[1] + 1;
    --private.state[2] = 0;
    
    local anm_hatch = function ()
      cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_insert_and_luke_opens");
      ObjSet("anm_breakthrough_hatch",  { playing = 1 });
    end;
    
    --ObjSet("anm_breakthrough_jean_"..private.state[3], { animfunc = "func_jim_stop_training_"..private.state[3], alp = 1, frame = 0, playing = 1, endtrig = private.NewLoc });
    ObjMultiSet( { {"anm_breakthrough_jean_"..private.state[3], { animfunc = "func_jim_stop_training_"..private.state[3], alp = 1, frame = 0, playing = 1, endtrig = function() private.NewLoc(); cmn.StopAllEnv(); end }},
                   {"tmr_breakthrough_timer_alien", { time = 2.3, playing = 1, endtrig = anm_hatch }},
                   {"spr_breakthrough_hatch_lock", { alp = 0 }} } );
    
  elseif private.state[1] > 0 and ingame then
    private.SetAlien();
  elseif ingame then
    ObjSet("anm_breakthrough_jean_"..private.state[3], { animfunc = "func_jim_stop_hologram_"..private.state[3], alp = 1, frame = 0, playing = 1, endtrig = private.SetJean });
  elseif private.state[1] > 0 and not ingame and private.state[4] == 0 then
    cmn.Lock( 1 );
    private.HeroDeath();
  end;

end;
-----------------------------------------------------------------------------------
function private.WinEnd()
  
  cmn.Lock( 0 );

  cmn.SetEventDone( "win_breakthrough" );
  
  if ng_global.currentprogress == "std" then 
  
    cmn.CallEventHandler( "win_breakthrough" );

    if ng_global.progress[ ng_global.currentprogress ][ "win_breakthrough" ].skip == 0 and not ng_global.noskipgames[ "win_breakthrough" ] then

      ng_global.noskipgames[ "win_breakthrough" ] = 1;
      common_impl.CheckAchiev("mg");

    end;
    
    if ng_global.playedgames and not ng_global.playedgames[ "win_breakthrough" ] then
      ng_global.playedgames[ "win_breakthrough" ] = 1;
    end;

    common_impl.PauseAchiev( 0 );
    
    cmn.GotoRoom("rm_hangar");
    
  else 
    
    ObjMultiSet( { {"grm_breakthrough_maindeck",  { input = 0 }},
                   {"obj_breakthrough_back",  { input = 0 }} } );
    
    SetCursor(CURSOR_DEFAULT);
    interface_impl.ShowStarsWindow();
    
  end;

end;
-----------------------------------------------------------------------------------
function private.Win()
  
  cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_mmg_monsters_win");
  
  if ng_global.currentprogress == "std" then

    cmn.MiniGameHide();

    common_impl.PauseAchiev( 1 );
    cmn.EndTimePassing( "mg_breakthrough" );
    
  else 
    rm_extra.MiniGameHide();
    int_dialog_secretroom.StopTimer();
  end;
  
  local anm_move = function ()
    
    cmn.PlayAudio("snd", "assets/levels/level2/audio/sfx/aud_mmg_monsters_steps");
    
    ObjAnimate( "obj_breakthrough_level_4", "scale_xy", 0, 0, private.WinEnd,
    {
      0.0, 3, 1, 1,
      1.6, 3, 1.3, 1.3
    } );
    
    ObjAnimate( "obj_breakthrough_level_4", "pos_y", 0, 0, "",
    {
      0.0, 3, 384,
      0.4, 3, 374,
      0.8, 3, 394,
      1.2, 3, 374,
      1.6, 3, 394
    } );
    
    ObjAnimate( "obj_breakthrough_interface", "pos_y", 0, 0, "",
    {
      0.0, 3, 384,
      0.4, 3, 374,
      0.8, 3, 394,
      1.2, 3, 374,
      1.6, 3, 394
    } );
    
  end;
  
  ObjSet("anm_breakthrough_door_open",  { playing = 1, endtrig = anm_move });

end;
--****************************************************************************************************************