-- name=ho_inboxes
--*********************************************************************************************************************
function public.Init()
  --------------------------------------------------------------------------------------------------
  room.Define( "ho_inboxes" );
  game.relations[ "ho_inboxes" ].exitroom = "rm_attic";
  private.mouse_active = false;
  --------------------------------------------------------------------------------------------------
  game.hint[ "win_inboxes" ] = 
  {
    type = "win",
    room = "rm_attic",
    zz = "ho_inboxes",
    zz_gate = "gho_attic_inboxes"
  };
  --------------------------------------------------------------------------------------------------
  local win_inboxes_func = function ()

    if ( cmn.IsEventDone( "win_inboxes" ) ) then
      interface.InventoryItemAdd( "inv_amethyst", "spr_inboxes_win" );
      int_map_impl.DecreaseRoomTasks("rm_attic");
      ObjSet("inv_egg_star_light",  { alp = 1 });
      ObjDelete("gho_attic_inboxes");
    end;

  end;
  cmn.AddSubscriber( "win_inboxes", win_inboxes_func );
  --------------------------------------------------------------------------------------------------
  local start_inboxes_func = function ()

    if ( cmn.IsEventStart( "win_inboxes" ) ) then

      common_impl.ProcessHoStart( "inboxes", true );

    end;

  end;
  cmn.AddSubscriber( "win_inboxes", start_inboxes_func, "ho_inboxes" );
  
  --for secretroom
  private.tasks =  {    
      { "badge",         1, 0, { { "spr_inboxes_badge" } } },
      { "screwdriver",   1, 0, { { "spr_inboxes_screwdriver", "obj_inboxes_screwdriver" } } },
      { "compass",       1, 0, { { "spr_inboxes_compass", "obj_inboxes_compass_1", "obj_inboxes_compass_2" } } },
      { "puffbox",       1, 0, { { "spr_inboxes_puffbox" } } },

      { "brush",     1, 0, { { "spr_inboxes_brush" } } },
      { "crowbar",   1, 0, { { "spr_inboxes_crowbar", "obj_inboxes_crowbar_1", "obj_inboxes_crowbar_2" } } },
      { "toy",       1, 0, { { "spr_inboxes_toy" } } },
      { "glove",     1, 0, { { "spr_inboxes_glove" } } },

      { "gem",       1, 0, { { "spr_inboxes_gem" } } },
      { "watch",     1, 0, { { "spr_inboxes_watch" } } },
      { "deer",      1, 0, { { "spr_inboxes_deer" } } },
      { "dragonfly", 1, 0, { { "spr_inboxes_dragonfly" } } },
      { "feltpen",   1, 0, { { "spr_inboxes_feltpen", "obj_inboxes_feltpen" } } }
  };
    
  private.tasks_sh =  { 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1 };
    
  private.start = {};
  private.start_sh = {};
  private.start_items = {};
  local nme = "";

  for i = 1, #private.tasks, 1 do

    for j = 1, #private.tasks[i][4][1], 1 do

      table.insert( private.start, {} );
      private.start[ #private.start ].name = private.tasks[ i ][ 4 ][ 1 ][j];
      nme = private.start[ #private.start ].name;
      private.start[ #private.start ].prop = ObjGet( nme );
      private.start[ #private.start ].parent = ObjGetRelations( nme ).parent;

    end;
    
    table.insert( private.start_items, {} );
    private.start_items[ #private.start_items ].name = "itm_"..common.GetObjectName( private.tasks[ i ][ 4 ][ 1 ][ 1 ] );
    nme = private.start_items[ #private.start_items ].name;
    private.start_items[ #private.start_items ].prop = ObjGet( nme );
  
    if private.tasks_sh[i] == 1 then
      
      table.insert( private.start_sh, {} );
      private.start_sh[ #private.start_sh ].name = private.tasks[ i ][ 4 ][ 1 ][ 1 ].."_shadow";
      nme = private.start_sh[ #private.start_sh ].name;
      private.start_sh[ #private.start_sh ].prop = ObjGet( nme );
      private.start_sh[ #private.start_sh ].parent = ObjGetRelations( nme ).parent;
      
    end;
    
  end;

end;
--*********************************************************************************************************************
function public.Load()

end;
--*********************************************************************************************************************
function public.PreOpen()
  
  cmn.StopAllEnv("assets/levels/level4/audio/env/aud_rm_attic_env");
  cmn.PlayAudio("env", "assets/levels/level4/audio/env/aud_rm_attic_env"); 

  if ng_global.currentprogress == "ext" then
    common.SetPreviousRoom( "rm_attic" );
    cmn.StartTimePassing("ho_inboxes");
  else 
    
    private.CreateNewObjects();
    
    interface.WidgetSetInput( InterfaceWidget_BtnHint,   true );
    interface.WidgetSetVisible( InterfaceWidget_BtnHint,   true, true );
    
    ObjSet("grm_inboxes_attic",  { event_mdown = "", event_mup = rm_extra.GoToSecretroom, input = 1,
                                   event_menter = function () rm_extra.GateMouseEnter( "inboxes_attic", CURSOR_DOWN, "extra" ); end, 
                                   event_mleave = function () common_impl.GateMouseLeave( "inboxes_attic" ); end });
    
  end;

  if not cmn.IsEventStart( "win_inboxes" ) then
    cmn.InitHo( "inboxes" );
  end;
  
  if not private.mouse_active then
    private.mouse_active = true;
    private.AnimMouse();
  end;

  local ho_items = {};

  interface_impl.ItemPanelItemAdd( ho_items, "badge", 0, "spr_inboxes_badge" );
  interface_impl.ItemPanelItemAdd( ho_items, "screwdriver", 0, "spr_inboxes_screwdriver" );
  interface_impl.ItemPanelItemAdd( ho_items, "compass", 0, "spr_inboxes_compass" );
  interface_impl.ItemPanelItemAdd( ho_items, "puffbox", 0, "spr_inboxes_puffbox" );
  
  interface_impl.ItemPanelItemAdd( ho_items, "brush", 0, "spr_inboxes_brush" );
  interface_impl.ItemPanelItemAdd( ho_items, "crowbar", 0, "spr_inboxes_crowbar" );
  interface_impl.ItemPanelItemAdd( ho_items, "toy", 0, "spr_inboxes_toy" );
  interface_impl.ItemPanelItemAdd( ho_items, "glove", 0, "spr_inboxes_glove" );
  
  interface_impl.ItemPanelItemAdd( ho_items, "gem", 0, "spr_inboxes_gem" );
  interface_impl.ItemPanelItemAdd( ho_items, "watch", 0, "spr_inboxes_watch" );
  interface_impl.ItemPanelItemAdd( ho_items, "deer", 0, "spr_inboxes_deer" );
  interface_impl.ItemPanelItemAdd( ho_items, "dragonfly", 0, "spr_inboxes_dragonfly" );
  interface_impl.ItemPanelItemAdd( ho_items, "feltpen", 0, "spr_inboxes_feltpen" );

  if (not cmn.IsEventDone("win_inboxes")) then
  
    interface_impl.ItemPanelShow( ho_items );
  
  else
    common_impl.DialogHo_Show();
  end;

end;
--*********************************************************************************************************************
function public.Open()    

  cmn.PlayAudio("mus", "assets/levels/level1/audio/mus/aud_bells_mus", 1);

end; 
--*********************************************************************************************************************
function public.PreClose()

  interface_impl.ItemPanelHide();
  
  cmn.StopAllMus();
  
  if ng_global.currentprogress == "ext" then
  
    common_impl.PlayBackgroundMusic();

    common_impl.StopwatchHoClose();
    common_impl.achiev.stopwatch = {};

    interface.WidgetSetInput( InterfaceWidget_Inventory, true );

    common_impl.PauseAchiev(0);
    
  else 
    
    ObjSet("grm_inboxes_attic",  { input = 0 });
    
    interface.WidgetSetInput( InterfaceWidget_BtnHint,   false );
    interface.WidgetSetVisible( InterfaceWidget_BtnHint,   false, true );
    
  end;

end;
--*********************************************************************************************************************
function public.HoComplete()
  
  if ng_global.currentprogress == "ext" then

    common_impl.PauseAchiev(1);

    cmn.EndTimePassing("ho_inboxes");

    if ng_global.progress[ ng_global.currentprogress ][ "win_inboxes" ].is_hint ~= 1
    and not ng_global.nohintho[ "win_inboxes" ] then

      ng_global.nohintho[ "win_inboxes" ] = 1;
      common_impl.CheckAchiev( "ho" );

    end;
    
    if ng_global.playedhops and not ng_global.playedhops[ "win_inboxes" ] then
      ng_global.playedhops[ "win_inboxes" ] = 1;
    end;
    
  else 
    
    ObjSet("grm_inboxes_attic",  { input = 0 });
    
  end;
  
end;
--*********************************************************************************************************************
function public.ItemFound( item_name )

  ObjDelete("spr_inboxes_"..item_name.."_shadow");

  cmn.HoItemFind( "spr_inboxes_"..item_name, 1 );
  
end;
-----------------------------------------------------------------------------------
function private.AnimMouse()
  
  local pause = math.random(10, 20);
  local mouse = math.random(1, 4);
  
  local anm_mouse = function ()
    
    ObjSet("anm_inboxes_mouse_"..mouse,  { playing = 1, endtrig = private.AnimMouse });
    
  end;
  
  ObjSet("tmr_inboxes_timer_mouse",  { time = pause, playing = 1, endtrig = anm_mouse });
  
end;
--*********************************************************************************************************************
function private.CreateNewObjects()

  local ho_obj = private.start;
  local ho_sh  = private.start_sh;
  local ho_itm = private.start_items;
  
  for i = 1, #ho_obj, 1 do
    
    if not ObjGet( ho_obj[i].name ) then
      
      local typ = common.GetObjectPrefix( ho_obj[i].name );
      
      if typ == "anm" then
        typ = "anim";
      end;
      
      ObjCreate( ho_obj[i].name, typ );
      ObjSet( ho_obj[i].name, ho_obj[i].prop );
      ObjAttach( ho_obj[i].name, ho_obj[i].parent );

    else 
      
      ObjSet( ho_obj[i].name, ho_obj[i].prop );
      ObjAttach( ho_obj[i].name, ho_obj[i].parent );

    end;

  end;

  for i = 1, #ho_sh, 1 do

    if not ObjGet( ho_sh[i].name ) then

      ObjCreate( ho_sh[i].name, "spr" );
      ObjSet( ho_sh[i].name, ho_sh[i].prop );
      ObjAttach( ho_sh[i].name, ho_sh[i].parent );

    end;

  end;
  
  for i = 1, #ho_itm, 1 do
    
    if not ObjGet( ho_itm[i].name ) then
      
      ObjCreate( ho_itm[i].name, "spr" );
      ObjSet( ho_itm[i].name, ho_itm[i].prop );
      ObjAttach( ho_itm[i].name, "obj_inboxes_items" );
      
    else 
      
      ObjSet( ho_itm[i].name, ho_itm[i].prop );
      ObjAttach( ho_itm[i].name, "obj_inboxes_items" );

    end;
    
  end;
  
end;
--*********************************************************************************************************************
